'''
作者:J.Hu
日期:2022-07-01
内容:
# 功能描述：
 - 支持邮件发送
# 重要版本变更说明：
 - 更新附件的代码编写,使其支持正常的文字显示;
 - To/Cc/Bcc的列表拼接采用逗号的形式拼接，原因是原来采用多个To/Cc/Bcc会导致
   在某些邮件客户端上只显示一个To/Cc/Bcc内容.
   比如：Header里面内容如：
   To:=?utf-8?b?6IOh5YGl?= <huj@citytsm.com>
   To:=?utf-8?b?6IOh5YGl?= <huj@citytsm.com>
   这样只会显示一个huj，
   现在修改变成：
   To:=?utf-8?b?6IOh5YGl?= <huj@citytsm.com>,=?utf-8?b?6IOh5YGl?= <huj@citytsm.com>
   这样就可以显示出2个huj，huj了
'''
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.mime.base import MIMEBase
from email.utils import formataddr
from email.header import Header
from smtplib import SMTP, SMTP_SSL
from pydantic import BaseModel
from typing import Optional

class MailKeyword(str):
    # 发送信关键字
    FROM = 'From'
    TO = 'To'
    CC = 'Cc'
    BCC = 'Bcc'
    TITLE = 'Subject'

    # 编码关键字
    ENCODE_UTF8 = 'utf-8'

    # 邮件类型关键字
    MAILTYPE_PLAIN = 'plain'
    MAILTYPE_HTML = 'html'
    MAILTYPE_BASE64 = 'base64'


class MailAttach(BaseModel):
    path: str
    name: Optional[str] = None

    def __init__(self, **kw):
        if kw.get('name') is None:
            kw['name'] = kw['path'].split('/')[-1]
        super().__init__(**kw)


class MailUser(BaseModel):
    addr: str
    name: Optional[str] = None

    def __init__(self, **kw):
        if kw.get('name') is None:
            kw['name'] = kw['addr']
        super().__init__(**kw)


class Mail:
    def __init__(self,
                 host: str,
                 port: int = 465,
                 ssl: bool = True,
                 timeout: int = 7,
                 encode: MailKeyword = MailKeyword.ENCODE_UTF8,
                 mail_type: MailKeyword = MailKeyword.MAILTYPE_HTML,
                 attach_type: MailKeyword = MailKeyword.MAILTYPE_BASE64):
        """
        host:smtp服务器
        port:smtp服务器端口号
        ssl: true:使用ssl
        timeout:超时时间,单位:秒
        encode:编码类型,默认为utf-8
        mail_type: 正文类型, 默认为html
        attach_type: 附件类型,默认为base64
        """

        self._smtp = self.__connect(host, port, ssl, timeout)
        self._encode = encode
        self._mail_type = mail_type
        self._attach_type = attach_type

        # 初始化to,cc,bcc list
        self._receive_lst = {}
        self.__receive_type_lst = [
            MailKeyword.TO, MailKeyword.CC, MailKeyword.BCC]

    @property
    def smtp(self):
        return self._smtp

    @property
    def encode(self):
        return self._encode

    @property
    def mail_type(self):
        return self._mail_type

    @property
    def attach_type(self):
        return self._attach_type

    @property
    def sender(self):
        return self._sender

    @property
    def receive_to(self):
        return self.__get_receive_lst(MailKeyword.TO)

    @property
    def receive_cc(self):
        return self.__get_receive_lst(MailKeyword.CC)

    @property
    def receive_bcc(self):
        return self.__get_receive_lst(MailKeyword.BCC)

    def __get_receive_lst(self, type: MailKeyword):
        return self._receive_lst.get(type, [])

    def __set_receiver_lst(self, type: MailKeyword, addr, name=None):
        u = MailUser(addr=addr, name=name)
        type_list = self.__get_receive_lst(type)
        if type_list == []:
            self._receive_lst[type] = [u]
        else:
            self._receive_lst[type].append(u)

    def __connect(self, host, port, ssl, timeout):
        smtpfunc = SMTP_SSL if ssl else SMTP
        return smtpfunc(host, port, timeout=timeout)

    def __mime_attach(self, mime, att, attid):
        li = MailAttach(path=att)
        att = MIMEText(open(li.path, 'rb').read(),
                       MailKeyword.MAILTYPE_BASE64, self.encode)
        att.add_header('Content-ID', str(attid))
        att.add_header('Content-Type', 'application/octet-stream')
        att.add_header("Content-Disposition", 'attachment',
                       filename=(MailKeyword.ENCODE_UTF8, '', li.name))
        mime.attach(att)

    def set_sender(self, addr, name=None):
        self._sender = MailUser(addr=addr, name=name)

    def add_receiver_to(self, addr, name=None):
        self.__set_receiver_lst(MailKeyword.TO, addr, name)

    def add_receiver_cc(self, addr, name=None):
        self.__set_receiver_lst(MailKeyword.CC, addr, name)

    def add_receiver_bcc(self, addr, name=None):
        self.__set_receiver_lst(MailKeyword.BCC, addr, name)

    def clear_receiver_lst(self):
        """
        清空收件人列表包含to,cc,bcc
        """
        self._receive_lst = {}

    def login(self, user: str, passwd: str, nickname: str = None):
        """
        user:登录用户名
        passwd:登录密码
        """
        self.smtp.login(user, passwd)
        self.set_sender(user, nickname)

    def send(self, title: str, content: str, attach_lst=None):
        """
        title:邮件标题
        content:邮件正文
        attach:邮件附件,默认为空
        """
        mime = MIMEMultipart()

        # 设置邮件标题,发件人
        mime[MailKeyword.TITLE] = Header(title, self.encode)
        mime[MailKeyword.FROM] = formataddr(
            [self.sender.name, self.sender.addr])

        # 设置收件人信息 to,cc,bcc
        receive_lst = []
        for val in self.__receive_type_lst:
            val_lst = self.__get_receive_lst(val)
            receive_lst.extend([mail.addr for mail in val_lst])
            mime[val] = ','.join(
                [formataddr([mail.name, mail.addr]) for mail in val_lst])

        # 正文内容设置
        mime.attach(MIMEText(content, self.mail_type, self.encode))

        # 附件内容设置
        for idx, val in enumerate(attach_lst):
            self.__mime_attach(mime, val, idx)

        # 发送邮件
        self.smtp.sendmail(self.sender.addr, receive_lst, mime.as_string())

    def disconnect(self):
        """
        关闭链接
        """
        self.smtp.close()


if __name__ == '__main__':
    server = 'smtp.qq.com'
    user = '344242312@qq.com'
    pwd = 'izzzrdislhwpbied'
    m = Mail(server)
    m.login(user, pwd, 'hello_world')

    attach = ['/Users/hujian/code/jhu/requirements.txt']

    m.add_receiver_to('*****@163.com', 'to_name')
    m.add_receiver_cc('*****@163.com', 'cc_name')
    m.add_receiver_bcc('*****@163.com')
    m.send('这是一次测试', '请注意这是一次测试演戏', attach)



"""
作者:J.Hu
日期:2022-06-24
内容:
> 自动分析系统并下载匹配最新的驱动,免除了手动下载匹配驱动
> 抽象化部分操作,通过简单的结构化配置即可实现脚本运行
"""
from selenium import webdriver
from requests import session
import json
import zipfile
import os
import platform


# 环境变量设定
DRIVER_NAME = 'chromedriver'
DRIVER_PATH = './' + DRIVER_NAME

# 淘宝镜像谷歌浏览器驱动下载地址
DRIVER_URL = 'https://registry.npmmirror.com/-/binary/chromedriver/'


def get_chromedriver_type() -> str:
    """根据系统类型获取下载驱动

    Returns
    -------
    'chromedriver_mac64.zip': Mac Intel驱动
    'chromedriver_mac64_m1.zip': Mac M系列驱动
    'chromedriver_linux64.zip': Linux驱动
    'chromedriver_win32.zip': Windows驱动
    """
    if platform.system() == 'Darwin':
        # x86_64架构视为非M系列 M系列为ARM架构
        return 'chromedriver_mac64.zip' if platform.machine() == 'x86_64' else 'chromedriver_mac64_m1.zip'
    elif platform.system() == 'Linux':
        return 'chromedriver_linux64.zip'
    else:
        return 'chromedriver_win32.zip'

def get_chromedriver_list(url: str) -> list:
    """获取可供下载的驱动版本

    Parameters
    ----------
    url:镜像驱动下载地址

    Returns
    -------
    {
        name:下载的驱动版本号
    }
    """
    with session() as conn:
        rsp = conn.get(url)
        chrome_list = json.loads(rsp.text)
    # 获取到的版本号为最后有一个/,舍弃掉,并且分割出版本数字,版本号中含有非数字的全都作为非版本数据处理掉,为后续版本比较处理,全部int化掉
    return [[int(i) for i in val['name'][:-1].split('.')] for val in chrome_list if val['name'][0].isdigit()]

def download_chrome_driver(url: str) -> None:
    """下载驱动文件并解压缩和设置权限
    Paramters
    ---------
    url:下载驱动的地址
    """
    driver_name = DRIVER_NAME
    zip_driver_name = driver_name + '.zip'
    with session() as conn:
        rsp = conn.get(url, stream=True)

    with open(zip_driver_name, 'wb') as f:
        for chunk in rsp.iter_content(chunk_size=1024):
            if chunk:
                f.write(chunk)
            else:
                break
    with zipfile.ZipFile(zip_driver_name) as zf:
        zf.extract(driver_name)
    os.chmod(driver_name, 766)

def get_chrome_version(message: str) -> str:
    """从错误信息中寻找浏览器的版本信息

    Parameters
    ----------
    message:出错信息

    Returns
    -------
    返回版本号信息,如果没有获取到版本号信息,返回空
    """
    start_msg = 'Current browser version is '
    end_msg = 'with binary path'

    end_idx = message.find(end_msg)
    start_idx = len(start_msg) + message.find(start_msg)
    if end_idx == -1:
        return ''
    return message[start_idx:end_idx].strip()

def fit_chrome_version(cur_version:str, check_list:list, prefix_url: str = DRIVER_URL) -> str:
    """根据当前的版本,匹配下载版本. 匹配的下载版本号不大于当前版本号,且是最接近的一个版本号

    Parameters
    ----------
    cur_version:当前版本信息
    check_list:可供下载的版本列表
    prefix_url:前置通用url

    Returns
    -------
    返回一个供下载的版本url
    """
    # 如果没有版本信息,返回一个默认值
    if cur_version == '':
        return prefix_url + '.'.join([str(val) for val in check_list[0]]) + '/'

    cur_list = [int(i) for i in cur_version.split('.')]
    cur_size = len(cur_list)
    version_info = ''
    for i in range(cur_size):
        ver_dif = min([cur_list[i] - val[i]
                       for val in check_list if cur_list[i] >= val[i]])
        check_list = [val for val in check_list if val[i]
                      == cur_list[i] - ver_dif]
        if i == 0:
            version_info = str(cur_list[i] - ver_dif)
        else:
            version_info = version_info + '.' + str(cur_list[i] - ver_dif)
    return prefix_url + version_info + '/'

def setup_dirver(cur_version:str, chrome_list:list):
    if not chrome_list:
        chrome_list = get_chromedriver_list(DRIVER_URL)
    download_url = fit_chrome_version(cur_version, chrome_list) + get_chromedriver_type()
    download_chrome_driver(download_url)
    return chrome_list



class WebdriveChrome:
    def __init__(self, driver_path=DRIVER_PATH):
        options = webdriver.ChromeOptions()
        # 忽略SSL验证
        options.add_argument('ignore-certificate-errors')
        # 伪装浏览器
        options.add_argument('User-Agent=Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/92.0.4515.159 Safari/537.36')

        # 屏蔽chrome正受到自动测试软件的控制
        options.add_experimental_option("excludeSwitches", ["enable-automation"])

        # 默认等待时间
        self._waittime = 5

        if not os.path.exists(driver_path):
            chrome_list = setup_dirver(cur_version,chrome_list)

        # 默认尝试3次.
        for _ in range(3):
            try:
                driver = webdriver.Chrome(executable_path=driver_path, options=options)
                self._driver = driver
                break
            except Exception as e:
                cur_version = get_chrome_version(e.msg)
                chrome_list = setup_dirver(cur_version, chrome_list)


    @property
    def driver(self):
        return self._driver

    @property
    def waittime(self):
        return self._waittime

if __name__ == '__main__':
    web = WebdriveChrome()
    web.driver.get('https://www.zjzwfw.gov.cn/zjservice/front/index/page.do?webId=1')



"""
作者:J.Hu
日期:2022-06-09
内容:
- 企业微信发送webhook消息,当前仅支持text格式
"""
from enum import Enum
from typing import Optional
from requests import Session


class WeComMsgType(str, Enum):
    txt = 'text'


class BaseContent:
    def __init__(self, **kw):
        self._data = kw

    @property
    def data(self) -> dict:
        return self._data

    @data.setter
    def data(self, data):
        self._data = data


class WeComText(BaseContent):
    def __init__(self, content: str, mentioned_list: Optional[list] = None, mentioned_mobile_list: Optional[list] = None):
        if len(content) > 2048:
            raise Exception('content内容长度不能超过2048字节')
        super().__init__(msgtype=WeComMsgType.txt.value, content=content,
                         mentioned_list=mentioned_list, mentioned_mobile_list=mentioned_mobile_list)


class WeComHook:
    def __init__(self, url: str):
        self.url = url

    def send(self, content: Optional[WeComText]):
        s = Session()
        rsp = s.post(self.url, json=content.data)
        rsp.encoding = rsp.apparent_encoding
        return rsp


if __name__ == '__main__':
    wh = WeComHook('******')
    t = WeComText("大家好,请", ['@all'])
    wh.send(t)



#! /usr/bin/env python3

"""
制作PIP包教程
1. 先设置一个setup文件
2. 运行setup.py sdist
3. 然后会得到一个dist文件, 这个时候就可用pip install dist/xxxxx 来安装文件了
4. twine upload dist/* 可上传安装,需要输入pypi的用户名和密码
"""
import setuptools
# from distutils.core import setup
from setuptools import setup

setup(
    name='jhu',  # 包的名称
    version='1.0.2',  # 包的版本,每次更新或升级包都需要更新它
    description='jhu是一个工具包,简化或自动化的做一些任务',  # 包的描述
    author='J.Hu',  # 作者
    email='joestarhu@163.com',  # 作者邮箱
    url='https://github.com/joestarhu/jhu',  # 项目地址
    packages=setuptools.find_packages(
        exclude=['test', 'examples', 'script', 'tutorials']),   # 包内不需要引用的文件
    classifiers=[
        "Environment :: Web Environment",
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: MacOS',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Programming Language :: Python :: 3.9',
    ],
    install_requires=['requests', 'pydantic'],  # 依赖的包
    zip_safe=True
)
